#!/usr/bin/env python
#coding: utf-8

from common import *

def test_pipeline():
    r = getconn()

    pipe = r.pipeline(transaction = False)

    pipe.set('a', 'a1').get('a').zadd('z', z1=1).zadd('z', z2=4)
    pipe.zincrby('z', 'z1').zrange('z', 0, 5, withscores=True)

    assert pipe.execute() == \
        [
            True,
            'a1',
            True,
            True,
            2.0,
            [('z1', 2.0), ('z2', 4)],
        ]

def test_invalid_pipeline():
    r = getconn()

    pipe = r.pipeline(transaction = False)

    pipe.set('a', 1).set('b', 2).lpush('a', 3).set('d', 4).get('a')
    result = pipe.execute(raise_on_error = False)

    assert result[0]
    assert result[1]

    # we can't lpush to a key that's a string value, so this should
    # be a ResponseError exception
    assert isinstance(result[2], redis.ResponseError)

    # since this isn't a transaction, the other commands after the
    # error are still executed
    assert result[3]
    assert result[4] == '1'

    # make sure the pipe was restored to a working state
    assert pipe.set('z', 'zzz').execute() == [True]

def test_parse_error_raised():
    r = getconn()

    pipe = r.pipeline(transaction = False)

    # the zrem is invalid because we don't pass any keys to it
    pipe.set('a', 1).zrem('b').set('b', 2)
    result = pipe.execute(raise_on_error = False)

    assert result[0]
    assert isinstance(result[1], redis.ResponseError)
    assert result[2]



#!/usr/bin/env python
#coding: utf-8

from common import *

def test_setget():
    r = getconn()

    rst = r.set('k', 'v')
    assert(r.get('k') == 'v')

def test_msetnx():
    r = getconn()

    #not supported
    keys = default_kv.keys()
    assert_fail('Socket closed|Connection closed', r.msetnx,**default_kv)

def test_null_key():
    r = getconn()
    rst = r.set('', 'v')
    assert(r.get('') == 'v')

    rst = r.set('', '')
    assert(r.get('') == '')

    kv = {'' : 'val', 'k': 'v'}
    ret = r.mset(**kv)
    assert(r.get('') == 'val')

def test_ping_quit():
    r = getconn()
    assert(r.ping() == True)

    #get set
    rst = r.set('k', 'v')
    assert(r.get('k') == 'v')

    assert_fail('Socket closed|Connection closed', r.execute_command, 'QUIT')

def test_slow_req():
    r = getconn()

    kv = {'mkkk-%s' % i : 'mvvv-%s' % i for i in range(500000)}

    pipe = r.pipeline(transaction=False)
    pipe.set('key-1', 'v1')
    pipe.get('key-1')
    pipe.hmset('xxx', kv)
    pipe.get('key-2')
    pipe.get('key-3')

    assert_fail('timed out', pipe.execute)

def test_signal():
    #init
    nc.cleanlog()
    nc.signal('HUP')

    nc.signal('HUP')
    nc.signal('TTIN')
    nc.signal('TTOU')
    nc.signal('SEGV')

    time.sleep(.3)
    log = file(nc.logfile()).read()

    assert(strstr(log, 'HUP'))
    assert(strstr(log, 'TTIN'))
    assert(strstr(log, 'TTOU'))
    assert(strstr(log, 'SEGV'))

    #recover
    nc.start()

def test_nc_stats():
    nc.stop() #reset counters
    nc.start()
    r = getconn()
    kv = {'kkk-%s' % i :'vvv-%s' % i for i in range(10)}
    for k, v in kv.items():
        r.set(k, v)
        r.get(k)

    def get_stat(name):
        time.sleep(1)
        stat = nc._info_dict()
        #pprint(stat)
        if name in ['client_connections', 'client_eof', 'client_err', \
                    'forward_error', 'fragments', 'server_ejects']:
            return stat[CLUSTER_NAME][name]

        #sum num of each server
        ret = 0
        for k, v in stat[CLUSTER_NAME].items():
            if type(v) == dict:
                ret += v[name]
        return ret

    assert(get_stat('requests') == 20)
    assert(get_stat('responses') == 20)

    ##### mget
    keys = kv.keys()
    r.mget(keys)

    #for version<=0.3.0
    #assert(get_stat('requests') == 30)
    #assert(get_stat('responses') == 30)

    #for mget-improve
    assert(get_stat('requests') == 22)
    assert(get_stat('responses') == 22)

def test_issue_323():
    # do on redis
    r = all_redis[0]
    c = redis.Redis(r.host(), r.port())
    assert([1, 'OK'] == c.eval("return {1, redis.call('set', 'x', '1')}", 1, 'tmp'))

    # do on twemproxy
    c = getconn()
    assert([1, 'OK'] == c.eval("return {1, redis.call('set', 'x', '1')}", 1, 'tmp'))

def setup_and_wait():
    time.sleep(60*60)




#!/usr/bin/env python
#coding: utf-8

from common import *

def test_mget_mset(kv=default_kv):
    r = getconn()

    def insert_by_pipeline():
        pipe = r.pipeline(transaction=False)
        for k, v in kv.items():
            pipe.set(k, v)
        pipe.execute()

    def insert_by_mset():
        ret = r.mset(**kv)

    #insert_by_mset() #only the mget-imporve branch support this
    try:
        insert_by_mset() #only the mget-imporve branch support this
    except:
        insert_by_pipeline()

    keys = kv.keys()

    #mget to check
    vals = r.mget(keys)
    for i, k in enumerate(keys):
        assert(kv[k] == vals[i])

    #del
    assert (len(keys) == r.delete(*keys) )

    #mget again
    vals = r.mget(keys)

    for i, k in enumerate(keys):
        assert(None == vals[i])

def test_mget_mset_on_key_not_exist(kv=default_kv):
    r = getconn()

    def insert_by_pipeline():
        pipe = r.pipeline(transaction=False)
        for k, v in kv.items():
            pipe.set(k, v)
        pipe.execute()

    def insert_by_mset():
        ret = r.mset(**kv)

    try:
        insert_by_mset() #only the mget-imporve branch support this
    except:
        insert_by_pipeline()

    keys = kv.keys()
    keys2 = ['x-'+k for k in keys]
    keys = keys + keys2
    random.shuffle(keys)

    #mget to check
    vals = r.mget(keys)
    for i, k in enumerate(keys):
        if k in kv:
            assert(kv[k] == vals[i])
        else:
            assert(vals[i] == None)

    #del
    assert (len(kv) == r.delete(*keys) )

    #mget again
    vals = r.mget(keys)

    for i, k in enumerate(keys):
        assert(None == vals[i])

def test_mget_mset_large():
    for cnt in range(171, large, 171):
        kv = {'kkk-%s' % i :'vvv-%s' % i for i in range(cnt)}
        test_mget_mset(kv)

def test_mget_special_key(cnt=5):
    #key length = 512-48-1
    kv = {}
    for i in range(cnt):
        k = 'kkk-%s' % i
        k = k + 'x'*(512-48-1-len(k))
        kv[k] = 'vvv'

    test_mget_mset(kv)

def test_mget_special_key_2(cnt=5):
    #key length = 512-48-2
    kv = {}
    for i in range(cnt):
        k = 'kkk-%s' % i
        k = k + 'x'*(512-48-2-len(k))
        kv[k] = 'vvv'*9

    test_mget_mset(kv)

def test_mget_on_backend_down():
    #one backend down

    r = redis.Redis(nc.host(), nc.port())
    assert_equal(None, r.get('key-2'))
    assert_equal(None, r.get('key-1'))

    all_redis[0].stop()

    assert_fail('Connection refused|reset by peer|Broken pipe', r.mget, 'key-1')
    assert_fail('Connection refused|reset by peer|Broken pipe', r.get, 'key-1')
    assert_equal(None, r.get('key-2'))

    keys = ['key-1', 'key-2', 'kkk-3']
    assert_fail('Connection refused|reset by peer|Broken pipe', r.mget, *keys)

    #all backend down
    all_redis[1].stop()
    r = redis.Redis(nc.host(), nc.port())

    assert_fail('Connection refused|reset by peer|Broken pipe', r.mget, 'key-1')
    assert_fail('Connection refused|reset by peer|Broken pipe', r.mget, 'key-2')

    keys = ['key-1', 'key-2', 'kkk-3']
    assert_fail('Connection refused|reset by peer|Broken pipe', r.mget, *keys)

    for r in all_redis:
        r.start()

def test_mset_on_backend_down():
    all_redis[0].stop()
    r = redis.Redis(nc.host(),nc.port())

    assert_fail('Connection refused|Broken pipe',r.mset,default_kv)

    all_redis[1].stop()
    assert_fail('Connection refused|Broken pipe',r.mset,default_kv)

    for r in all_redis:
        r.start()

def test_mget_pipeline():
    r = getconn()

    pipe = r.pipeline(transaction=False)
    for k,v in default_kv.items():
        pipe.set(k,v)
    keys = default_kv.keys()
    pipe.mget(keys)
    kv = {}
    for i in range(large):
        kv['kkk-%s' % i] = os.urandom(100)
    for k,v in kv.items():
        pipe.set(k,v)
    for k in kv.keys():
        pipe.get(k)
    rst = pipe.execute()

    #print rst
    #check the result
    keys = default_kv.keys()

    #mget to check
    vals = r.mget(keys)
    for i, k in enumerate(keys):
        assert(kv[k] == vals[i])

    #del
    assert (len(keys) == r.delete(*keys) )

    #mget again
    vals = r.mget(keys)

    for i, k in enumerate(keys):
        assert(None == vals[i])

def test_multi_delete_normal():
    r = getconn()

    for i in range(100):
        r.set('key-%s'%i, 'val-%s'%i)
    for i in range(100):
        assert_equal('val-%s'%i, r.get('key-%s'%i) )

    keys = ['key-%s'%i for i in range(100)]
    assert_equal(100, r.delete(*keys))

    for i in range(100):
        assert_equal(None, r.get('key-%s'%i) )

def test_multi_delete_on_readonly():
    all_redis[0].slaveof(all_redis[1].args['host'], all_redis[1].args['port'])

    r = redis.Redis(nc.host(), nc.port())

    # got "READONLY You can't write against a read only slave"
    assert_fail('READONLY|Invalid', r.delete, 'key-1')
    assert_equal(0, r.delete('key-2'))
    assert_fail('READONLY|Invalid', r.delete, 'key-3')

    keys = ['key-1', 'key-2', 'kkk-3']
    assert_fail('Invalid argument', r.delete, *keys)     # got "Invalid argument"

def test_multi_delete_on_backend_down():
    #one backend down
    all_redis[0].stop()
    r = redis.Redis(nc.host(), nc.port())

    assert_fail('Connection refused|reset by peer|Broken pipe', r.delete, 'key-1')
    assert_equal(None, r.get('key-2'))

    keys = ['key-1', 'key-2', 'kkk-3']
    assert_fail('Connection refused|reset by peer|Broken pipe', r.delete, *keys)

    #all backend down
    all_redis[1].stop()
    r = redis.Redis(nc.host(), nc.port())

    assert_fail('Connection refused|reset by peer|Broken pipe', r.delete, 'key-1')
    assert_fail('Connection refused|reset by peer|Broken pipe', r.delete, 'key-2')

    keys = ['key-1', 'key-2', 'kkk-3']
    assert_fail('Connection refused|reset by peer|Broken pipe', r.delete, *keys)

    for r in all_redis:
        r.start()


def test_multi_delete_20140525():
    r = getconn()

    cnt = 126
    keys = ['key-%s'%i for i in range(cnt)]
    pipe = r.pipeline(transaction=False)
    pipe.mget(keys)
    pipe.delete(*keys)
    pipe.execute()





#!/usr/bin/env python
#coding: utf-8
#file   : test_reload.py
#author : ning
#date   : 2014-09-03 12:28:16

import os
import sys
import redis

PWD = os.path.dirname(os.path.realpath(__file__))
WORKDIR = os.path.join(PWD,'../')
sys.path.append(os.path.join(WORKDIR,'lib/'))
sys.path.append(os.path.join(WORKDIR,'conf/'))

import conf

from server_modules import *
from utils import *
from nose import with_setup

CLUSTER_NAME = 'ntest'
nc_verbose = int(getenv('T_VERBOSE', 5))
mbuf = int(getenv('T_MBUF', 512))
large = int(getenv('T_LARGE', 1000))

T_RELOAD_DELAY = 3 + 1

all_redis = [
        RedisServer('127.0.0.1', 2100, '/tmp/r/redis-2100/', CLUSTER_NAME, 'redis-2100'),
        RedisServer('127.0.0.1', 2101, '/tmp/r/redis-2101/', CLUSTER_NAME, 'redis-2101'),
    ]

nc = NutCracker('127.0.0.1', 4100, '/tmp/r/nutcracker-4100', CLUSTER_NAME,
                all_redis, mbuf=mbuf, verbose=nc_verbose)

def _setup():
    print 'setup(mbuf=%s, verbose=%s)' %(mbuf, nc_verbose)
    for r in all_redis + [nc]:
        r.deploy()
        r.stop()
        r.start()

def _teardown():
    for r in all_redis + [nc]:
        assert(r._alive())
        r.stop()

def get_tcp_conn(host, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((host, port))
    s.settimeout(.3)
    return s

def send_cmd(s, req, resp):
    s.sendall(req)
    data = s.recv(10000)
    assert(data == resp)

@with_setup(_setup, _teardown)
def test_reload_with_old_conf():
    if nc.version() < '0.4.2':
        print 'Ignore test_reload for version %s' % nc.version()
        return
    pid = nc.pid()
    # print 'old pid:', pid
    r = redis.Redis(nc.host(), nc.port())
    r.set('k', 'v')

    conn = get_tcp_conn(nc.host(), nc.port())
    send_cmd(conn, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '$1\r\nv\r\n')

    # nc.reload() is same as nc.stop() and nc.start()
    nc.reload()
    time.sleep(.01)  #it need time for the old process fork new process.

    # the old connection is still ok in T_RELOAD_DELAY seconds
    send_cmd(conn, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '$1\r\nv\r\n')

    # conn2 should connect to new instance
    conn2 = get_tcp_conn(nc.host(), nc.port())
    send_cmd(conn2, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '$1\r\nv\r\n')

    # the old connection is still ok in T_RELOAD_DELAY seconds
    send_cmd(conn, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '$1\r\nv\r\n')

    time.sleep(T_RELOAD_DELAY)
    assert(pid != nc.pid())

    # assert the old connection is closed.
    send_cmd(conn, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '')

    # conn2 should survive
    send_cmd(conn2, '*2\r\n$3\r\nGET\r\n$1\r\nk\r\n', '$1\r\nv\r\n')

    r = redis.Redis(nc.host(), nc.port())
    rst = r.set('k', 'v')
    assert(r.get('k') == 'v')

@with_setup(_setup, _teardown)
def test_new_port():
    if nc.version() < '0.4.2':
        print 'Ignore test_reload for version %s' % nc.version()
        return
    r = redis.Redis(nc.host(), nc.port())
    r.set('k', 'v')

    content = '''
reload_test:
  listen: 0.0.0.0:4101
  hash: fnv1a_64
  distribution: modula
  redis: true
  timeout: 400
  servers:
    - 127.0.0.1:2100:1 redis-2100
    - 127.0.0.1:2101:1 redis-2101
'''

    nc.set_config(content)
    time.sleep(T_RELOAD_DELAY)

    r1 = redis.Redis(nc.host(), nc.port())
    r2 = redis.Redis(nc.host(), 4101)

    assert_fail('Connection refused', r1.get, 'k')
    assert(r2.get('k') == 'v')

@with_setup(_setup, _teardown)
def test_pool_add_del():
    if nc.version() < '0.4.2':
        print 'Ignore test_reload for version %s' % nc.version()
        return

    r = redis.Redis(nc.host(), nc.port())

    r.set('k', 'v')
    content = '''
reload_test:
  listen: 0.0.0.0:4100
  hash: fnv1a_64
  distribution: modula
  redis: true
  servers:
    - 127.0.0.1:2100:1 redis-2100
    - 127.0.0.1:2101:1 redis-2101

reload_test2:
  listen: 0.0.0.0:4101
  hash: fnv1a_64
  distribution: modula
  redis: true
  servers:
    - 127.0.0.1:2100:1 redis-2100
    - 127.0.0.1:2101:1 redis-2101
'''

    nc.set_config(content)
    time.sleep(T_RELOAD_DELAY)

    r1 = redis.Redis(nc.host(), nc.port())
    r2 = redis.Redis(nc.host(), 4101)

    assert(r1.get('k') == 'v')
    assert(r2.get('k') == 'v')

    content = '''
reload_test:
  listen: 0.0.0.0:4102
  hash: fnv1a_64
  distribution: modula
  redis: true
  preconnect: true
  servers:
    - 127.0.0.1:2100:1 redis-2100
    - 127.0.0.1:2101:1 redis-2101
'''
    nc.set_config(content)
    time.sleep(T_RELOAD_DELAY)
    pid = nc.pid()
    print system('ls -l /proc/%s/fd/' % pid)

    r3 = redis.Redis(nc.host(), 4102)

    assert_fail('Connection refused', r1.get, 'k')
    assert_fail('Connection refused', r2.get, 'k')
    assert(r3.get('k') == 'v')

    fds = system('ls -l /proc/%s/fd/' % pid)
    sockets = [s for s in fds.split('\n') if strstr(s, 'socket:') ]
    # pool + stat + 2 backend + 1 client
    assert(len(sockets) == 5)




#!/usr/bin/env python
from common import *
from pprint import pprint

def get_conn():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((nc.host(), nc.port()))
    s.settimeout(.3)
    return s

def _test(req, resp, sleep=0):
    s = get_conn()

    for i in req:
        s.sendall(i)
        time.sleep(sleep)

    s.settimeout(.3)

    data = s.recv(10000)
    assert(data == resp)

def test_slow():
    req = '*1\r\n$4\r\nPING\r\n'
    resp = '+PONG\r\n'

    if large > 1000:
        sleep = 1
    else:
        sleep = .1

    _test(req, resp, sleep)

def test_pingpong():
    req = '*1\r\n$4\r\nPING\r\n'
    resp = '+PONG\r\n'
    _test(req, resp)

def test_quit():
    if nc.version() < '0.4.2':
        return
    req = '*1\r\n$4\r\nQUIT\r\n'
    resp = '+OK\r\n'
    _test(req, resp)

def test_quit_without_recv():
    if nc.version() < '0.4.2':
        return
    req = '*1\r\n$4\r\nQUIT\r\n'
    resp = '+OK\r\n'
    s = get_conn()

    s.sendall(req)
    s.close()
    info = nc._info_dict()
    #pprint(info)
    assert(info['ntest']['client_err'] == 1)

def _test_bad(req):
    s = get_conn()

    s.sendall(req)
    data = s.recv(10000)
    print data

    assert('' == s.recv(1000))  # peer is closed

def test_badreq():
    reqs = [
        # '*1\r\n$3\r\nPING\r\n',
        '\r\n',
        # '*3abcdefg\r\n',
        '*3\r\n*abcde\r\n',

        '*4\r\n$4\r\nMSET\r\n$1\r\nA\r\n$1\r\nA\r\n$1\r\nA\r\n',
        '*2\r\n$4\r\nMSET\r\n$1\r\nA\r\n',
        # '*3\r\n$abcde\r\n',
        # '*3\r\n$3abcde\r\n',
        # '*3\r\n$3\r\nabcde\r\n',
    ]

    for req in reqs:
        _test_bad(req)


def test_wrong_argc():
    s = get_conn()

    s.sendall('*1\r\n$3\r\nGET\r\n')
    assert('' == s.recv(1000))  # peer is closed



#!/usr/bin/env python
#coding: utf-8
#file   : server_modules.py
#author : ning
#date   : 2014-02-24 13:00:28

import os
import sys

from utils import *
import conf

class Base:
    '''
    Sub class should implement:
    _alive, _pre_deploy, status, and init self.args
    '''
    def __init__(self, name, host, port, path):
        self.args = {
            'name'      : name,
            'host'      : host,
            'port'      : port,
            'path'      : path,

            #startcmd and runcmd will used to generate the control script
            #used for the start cmd
            'startcmd'  : '',
            #process name you see in `ps -aux`, used this to generate stop cmd
            'runcmd'    : '',
            'logfile'   : '',
        }

    def __str__(self):
        return TT('[$name:$host:$port]', self.args)

    def deploy(self):
        logging.info('deploy %s' % self)
        self._run(TTCMD('mkdir -p $path/bin &&  \
                      mkdir -p $path/conf && \
                      mkdir -p $path/log &&  \
                      mkdir -p $path/data',
                self.args))

        self._pre_deploy()
        self._gen_control_script()

    def _gen_control_script(self):
        content = file(os.path.join(WORKDIR, 'conf/control.sh')).read()
        content = TT(content, self.args)

        control_filename = TT('${path}/${name}_control', self.args)

        fout = open(control_filename, 'w+')
        fout.write(content)
        fout.close()
        os.chmod(control_filename, 0755)

    def start(self):
        if self._alive():
            logging.warn('%s already running' %(self) )
            return

        logging.debug('starting %s' % self)
        t1 = time.time()
        sleeptime = .1

        cmd = TT("cd $path && ./${name}_control start", self.args)
        self._run(cmd)

        while not self._alive():
            lets_sleep(sleeptime)
            if sleeptime < 5:
                sleeptime *= 2
            else:
                sleeptime = 5
                logging.warn('%s still not alive' % self)

        t2 = time.time()
        logging.info('%s start ok in %.2f seconds' %(self, t2-t1) )

    def stop(self):
        if not self._alive():
            logging.warn('%s already stop' %(self) )
            return

        cmd = TT("cd $path && ./${name}_control stop", self.args)
        self._run(cmd)

        t1 = time.time()
        while self._alive():
            lets_sleep()
        t2 = time.time()
        logging.info('%s stop ok in %.2f seconds' %(self, t2-t1) )

    def pid(self):
        cmd = TT("pgrep -f '^$runcmd'", self.args)
        return self._run(cmd)

    def status(self):
        logging.warn("status: not implement")

    def _alive(self):
        logging.warn("_alive: not implement")

    def _run(self, raw_cmd):
        ret = system(raw_cmd, logging.debug)
        logging.debug('return : [%d] [%s] ' % (len(ret), shorten(ret)) )
        return ret

    def clean(self):
        cmd = TT("rm -rf $path", self.args)
        self._run(cmd)

    def host(self):
        return self.args['host']

    def port(self):
        return self.args['port']

class RedisServer(Base):
    def __init__(self, host, port, path, cluster_name, server_name, auth = None):
        Base.__init__(self, 'redis', host, port, path)

        self.args['startcmd']     = TT('bin/redis-server conf/redis.conf', self.args)
        self.args['runcmd']       = TT('redis-server \*:$port', self.args)
        self.args['conf']         = TT('$path/conf/redis.conf', self.args)
        self.args['pidfile']      = TT('$path/log/redis.pid', self.args)
        self.args['logfile']      = TT('$path/log/redis.log', self.args)
        self.args['dir']          = TT('$path/data', self.args)
        self.args['REDIS_CLI']    = conf.BINARYS['REDIS_CLI']

        self.args['cluster_name'] = cluster_name
        self.args['server_name']  = server_name
        self.args['auth']         = auth

    def _info_dict(self):
        cmd = TT('$REDIS_CLI -h $host -p $port INFO', self.args)
        if self.args['auth']:
            cmd = TT('$REDIS_CLI -h $host -p $port -a $auth INFO', self.args)
        info = self._run(cmd)

        info = [line.split(':', 1) for line in info.split('\r\n')
                if not line.startswith('#')]
        info = [i for i in info if len(i) > 1]
        return defaultdict(str, info) #this is a defaultdict, be Notice

    def _ping(self):
        cmd = TT('$REDIS_CLI -h $host -p $port PING', self.args)
        if self.args['auth']:
            cmd = TT('$REDIS_CLI -h $host -p $port -a $auth PING', self.args)
        return self._run(cmd)

    def _alive(self):
        return strstr(self._ping(), 'PONG')

    def _gen_conf(self):
        content = file(os.path.join(WORKDIR, 'conf/redis.conf')).read()
        content = TT(content, self.args)
        if self.args['auth']:
            content += '\r\nrequirepass %s' % self.args['auth']
        return content

    def _pre_deploy(self):
        self.args['BINS'] = conf.BINARYS['REDIS_SERVER_BINS']
        self._run(TT('cp $BINS $path/bin/', self.args))

        fout = open(TT('$path/conf/redis.conf', self.args), 'w+')
        fout.write(self._gen_conf())
        fout.close()

    def status(self):
        uptime = self._info_dict()['uptime_in_seconds']
        if uptime:
            logging.info('%s uptime %s seconds' % (self, uptime))
        else:
            logging.error('%s is down' % self)

    def isslaveof(self, master_host, master_port):
        info = self._info_dict()
        if info['master_host'] == master_host and \
           int(info['master_port']) == master_port:
            logging.debug('already slave of %s:%s' % (master_host, master_port))
            return True

    def slaveof(self, master_host, master_port):
        cmd = 'SLAVEOF %s %s' % (master_host, master_port)
        return self.rediscmd(cmd)

    def rediscmd(self, cmd):
        args = copy.deepcopy(self.args)
        args['cmd'] = cmd
        cmd = TT('$REDIS_CLI -h $host -p $port $cmd', args)
        logging.info('%s %s' % (self, cmd))
        return self._run(cmd)

class Memcached(Base):
    def __init__(self, host, port, path, cluster_name, server_name):
        Base.__init__(self, 'memcached', host, port, path)

        self.args['startcmd']     = TT('bin/memcached -d -p $port', self.args)
        self.args['runcmd']       = self.args['startcmd']

        self.args['cluster_name'] = cluster_name
        self.args['server_name']  = server_name

    def _alive(self):
        cmd = TT('echo "stats" | socat - TCP:$host:$port', self.args)
        ret = self._run(cmd)
        return strstr(ret, 'END')

    def _pre_deploy(self):
        self.args['BINS'] = conf.BINARYS['MEMCACHED_BINS']
        self._run(TT('cp $BINS $path/bin/', self.args))

class NutCracker(Base):
    def __init__(self, host, port, path, cluster_name, masters, mbuf=512,
            verbose=5, is_redis=True, redis_auth=None):
        Base.__init__(self, 'nutcracker', host, port, path)

        self.masters = masters

        self.args['mbuf']        = mbuf
        self.args['verbose']     = verbose
        self.args['redis_auth']  = redis_auth
        self.args['conf']        = TT('$path/conf/nutcracker.conf', self.args)
        self.args['pidfile']     = TT('$path/log/nutcracker.pid', self.args)
        self.args['logfile']     = TT('$path/log/nutcracker.log', self.args)
        self.args['status_port'] = self.args['port'] + 1000

        self.args['startcmd'] = TTCMD('bin/nutcracker -d -c $conf -o $logfile \
                                       -p $pidfile -s $status_port            \
                                       -v $verbose -m $mbuf -i 1', self.args)
        self.args['runcmd']   = TTCMD('bin/nutcracker -d -c $conf -o $logfile \
                                       -p $pidfile -s $status_port', self.args)

        self.args['cluster_name']= cluster_name
        self.args['is_redis']= str(is_redis).lower()

    def _alive(self):
        return self._info_dict()

    def _gen_conf_section(self):
        template = '    - $host:$port:1 $server_name'
        cfg = '\n'.join([TT(template, master.args) for master in self.masters])
        return cfg

    def _gen_conf(self):
        content = '''
$cluster_name:
  listen: 0.0.0.0:$port
  hash: fnv1a_64
  distribution: modula
  preconnect: true
  auto_eject_hosts: false
  redis: $is_redis
  backlog: 512
  timeout: 400
  client_connections: 0
  server_connections: 1
  server_retry_timeout: 2000
  server_failure_limit: 2
  servers:
'''
        if self.args['redis_auth']:
            content = content.replace('redis: $is_redis',
                    'redis: $is_redis\r\n  redis_auth: $redis_auth')
        content = TT(content, self.args)
        return content + self._gen_conf_section()

    def _pre_deploy(self):
        self.args['BINS'] = conf.BINARYS['NUTCRACKER_BINS']
        self._run(TT('cp $BINS $path/bin/', self.args))

        fout = open(TT('$path/conf/nutcracker.conf', self.args), 'w+')
        fout.write(self._gen_conf())
        fout.close()

    def version(self):
        #This is nutcracker-0.4.0
        s = self._run(TT('$BINS --version', self.args))
        return s.strip().replace('This is nutcracker-', '')

    def _info_dict(self):
        try:
            c = telnetlib.Telnet(self.args['host'], self.args['status_port'])
            ret = c.read_all()
            return json_decode(ret)
        except Exception, e:
            logging.debug('can not get _info_dict of nutcracker, \
                          [Exception: %s]' % (e, ))
            return None

    def reconfig(self, masters):
        self.masters = masters
        self.stop()
        self.deploy()
        self.start()
        logging.info('proxy %s:%s is updated' % (self.args['host'], self.args['port']))

    def logfile(self):
        return self.args['logfile']

    def cleanlog(self):
        cmd = TT("rm '$logfile'", self.args)
        self._run(cmd)

    def signal(self, signo):
        self.args['signo'] = signo
        cmd = TT("pkill -$signo -f '^$runcmd'", self.args)
        self._run(cmd)

    def reload(self):
        self.signal('USR1')

    def set_config(self, content):
        fout = open(TT('$path/conf/nutcracker.conf', self.args), 'w+')
        fout.write(content)
        fout.close()

        self.reload()




#coding: utf-8

import os
import sys

PWD = os.path.dirname(os.path.realpath(__file__))
WORKDIR = os.path.join(PWD,  '../')

BINARYS = {
    'REDIS_SERVER_BINS'   : os.path.join(WORKDIR, '_binaries/redis-*'),
    'REDIS_CLI'           : os.path.join(WORKDIR, '_binaries/redis-cli'),
    'MEMCACHED_BINS'      : os.path.join(WORKDIR, '_binaries/memcached'),
    'NUTCRACKER_BINS'     : os.path.join(WORKDIR, '_binaries/nutcracker'),
}




#!/usr/bin/env python
#coding: utf-8

from common import *
from test_mget_mset import test_mget_mset as _mget_mset

#force to use large mbuf, we need to copy the setup/teardown here..

mbuf = 64*1024

nc = NutCracker(nc.host(), nc.port(), '/tmp/r/nutcracker-4100', CLUSTER_NAME,
                all_redis, mbuf=mbuf, verbose=nc_verbose)

def setup():
    print 'special setup(mbuf=%s, verbose=%s)' %(mbuf, nc_verbose)
    for r in all_redis + [nc]:
        r.deploy()
        r.stop()
        r.start()

def teardown():
    for r in all_redis + [nc]:
        assert(r._alive())
        r.stop()

######################################################
def test_mget_binary_value(cnt=5):
    kv = {}
    for i in range(cnt):
        kv['kkk-%s' % i] = os.urandom(1024*1024*16+1024) #16M
    for i in range(cnt):
        kv['kkk2-%s' % i] = ''
    _mget_mset(kv)




#!/usr/bin/env python
#coding: utf-8

from common import *

def test_linsert():
    r = getconn()

    r.rpush('mylist', 'Hello')
    r.rpush('mylist', 'World')
    r.linsert('mylist', 'BEFORE', 'World', 'There')

    rst = r.lrange('mylist', 0, -1)
    assert(rst == ['Hello', 'There', 'World'])

def test_lpush_lrange():
    r = getconn()

    vals = ['vvv-%s' % i for i in range(10) ]
    assert([] == r.lrange('mylist', 0, -1))

    r.lpush('mylist', *vals)
    rst = r.lrange('mylist', 0, -1)

    assert(10 == len(rst))

def test_hscan():
    r = getconn()

    kv = {'kkk-%s' % i : 'vvv-%s' % i for i in range(10)}
    r.hmset('a', kv)

    cursor, dic = r.hscan('a')
    assert(str(cursor) == '0')
    assert(dic == kv)

    cursor, dic = r.hscan('a', match='kkk-5')
    assert(str(cursor) == '0')
    assert(dic == {'kkk-5': 'vvv-5'})

def test_hscan_large():
    r = getconn()

    kv = {'x'* 100 + 'kkk-%s' % i : 'vvv-%s' % i for i in range(1000)}
    r.hmset('a', kv)

    cursor = '0'
    dic = {}
    while True:
        cursor, t = r.hscan('a', cursor, count=10)
        for k, v in t.items():
            dic[k] = v

        if '0' == str(cursor):
            break

    assert(dic == kv)

    cursor, dic = r.hscan('a', '0', match='*kkk-5*', count=1000)
    if str(cursor) == '0':
        assert(len(dic) == 111)
    else:
        assert(len(dic) == 111)

        #again.
        cursor, dic = r.hscan('a', cursor, match='*kkk-5*', count=1000)
        assert(str(cursor) == '0')
        assert(len(dic) == 0)

def test_zscan():
    r = getconn()

    r.zadd('a', 'a', 1, 'b', 2, 'c', 3)

    cursor, pairs = r.zscan('a')
    assert(str(cursor) == '0')
    assert(set(pairs) == set([('a', 1), ('b', 2), ('c', 3)]))

    cursor, pairs = r.zscan('a', match='a')
    assert(str(cursor) == '0')
    assert(set(pairs) == set([('a', 1)]))

def test_sscan():
    r = getconn()

    r.sadd('a', 1, 2, 3)

    cursor, members = r.sscan('a')
    assert(str(cursor) == '0')
    assert(set(members) == set(['1', '2', '3']))

    cursor, members = r.sscan('a', match='1')
    assert(str(cursor) == '0')
    assert(set(members) == set(['1']))




import os
import re
import sys
import time
import copy
import thread
import socket
import threading
import logging
import inspect
import argparse
import telnetlib
import redis
import random
import redis
import json
import glob
import commands

from collections import defaultdict
from argparse import RawTextHelpFormatter

from string import Template

PWD = os.path.dirname(os.path.realpath(__file__))
WORKDIR = os.path.join(PWD,  '../')

def getenv(key, default):
    if key in os.environ:
        return os.environ[key]
    return default

logfile = getenv('T_LOGFILE', 'log/t.log')
if logfile == '-':
    logging.basicConfig(level=logging.DEBUG,
        format="%(asctime)-15s [%(threadName)s] [%(levelname)s] %(message)s")
else:
    logging.basicConfig(filename=logfile, level=logging.DEBUG,
        format="%(asctime)-15s [%(threadName)s] [%(levelname)s] %(message)s")

logging.info("test running")

def strstr(s1, s2):
    return s1.find(s2) != -1

def lets_sleep(SLEEP_TIME = 0.1):
    time.sleep(SLEEP_TIME)

def TT(template, args): #todo: modify all
    return Template(template).substitute(args)

def TTCMD(template, args): #todo: modify all
    '''
    Template for cmd (we will replace all spaces)
    '''
    ret = TT(template, args)
    return re.sub(' +', ' ', ret)

def nothrow(ExceptionToCheck=Exception, logger=None):
    def deco_retry(f):
        def f_retry(*args, **kwargs):
            try:
                return f(*args, **kwargs)
            except ExceptionToCheck, e:
                if logger:
                    logger.info(e)
                else:
                    print str(e)
        return f_retry  # true decorator
    return deco_retry

@nothrow(Exception)
def test_nothrow():
    raise Exception('exception: xx')

def json_encode(j):
    return json.dumps(j, indent=4, cls=MyEncoder)

def json_decode(j):
    return json.loads(j)

#commands dose not work on windows..
def system(cmd, log_fun=logging.info):
    if log_fun: log_fun(cmd)
    r = commands.getoutput(cmd)
    return r

def shorten(s, l=80):
    if len(s)<=l:
        return s
    return s[:l-3] + '...'

def assert_true(a):
    assert a, 'assert fail: except true, got %s' % a

def assert_equal(a, b):
    assert a == b, 'assert fail: %s vs %s' % (shorten(str(a)), shorten(str(b)))

def assert_raises(exception_cls, callable, *args, **kwargs):
    try:
        callable(*args, **kwargs)
    except exception_cls as e:
        return e
    except Exception as e:
        assert False, 'assert_raises %s but raised: %s' % (exception_cls, e)
    assert False, 'assert_raises %s but nothing raise' % (exception_cls)

def assert_fail(err_response, callable, *args, **kwargs):
    try:
        callable(*args, **kwargs)
    except Exception as e:
        assert re.search(err_response, str(e)), \
               'assert "%s" but got "%s"' % (err_response, e)
        return

    assert False, 'assert_fail %s but nothing raise' % (err_response)

if __name__ == "__main__":
    test_nothrow()



import redis

range=100
factor=32
port=22121

r = redis.StrictRedis(host='localhost', port=port, db=0)

# lrange
print [r.lrange('lfoo', 0, x) for x in xrange(1, range)]
print [r.lpush('lfoo', str(x)*factor) for x in xrange(1, range)]
print [r.lrange('lfoo', 0, x) for x in xrange(1, range)]
print r.delete('lfoo')

# del
print [r.set('foo' + str(x), str(x)*factor) for x in xrange(1, range)]
keys = ['foo' + str(x) for x in xrange(1, range)]
print [r.delete(keys) for x in xrange(1, range)]

# mget
print [r.set('foo' + str(x), str(x)*100) for x in xrange(1, range)]
keys = ['foo' + str(x) for x in xrange(1, range)]
print [r.mget(keys) for x in xrange(1, range)]



#!/usr/bin/env python
#coding: utf-8

import os
import sys
import redis

PWD = os.path.dirname(os.path.realpath(__file__))
WORKDIR = os.path.join(PWD,'../')
sys.path.append(os.path.join(WORKDIR,'lib/'))
sys.path.append(os.path.join(WORKDIR,'conf/'))

import conf

from server_modules import *
from utils import *

CLUSTER_NAME = 'ntest'
nc_verbose = int(getenv('T_VERBOSE', 5))
mbuf = int(getenv('T_MBUF', 512))
large = int(getenv('T_LARGE', 1000))

all_redis = [
        RedisServer('127.0.0.1', 2100, '/tmp/r/redis-2100/', CLUSTER_NAME, 'redis-2100'),
        RedisServer('127.0.0.1', 2101, '/tmp/r/redis-2101/', CLUSTER_NAME, 'redis-2101'),
        ]

nc = NutCracker('127.0.0.1', 4100, '/tmp/r/nutcracker-4100', CLUSTER_NAME,
                all_redis, mbuf=mbuf, verbose=nc_verbose)

def setup():
    print 'setup(mbuf=%s, verbose=%s)' %(mbuf, nc_verbose)
    for r in all_redis + [nc]:
        r.clean()
        r.deploy()
        r.stop()
        r.start()

def teardown():
    for r in all_redis + [nc]:
        assert(r._alive())
        r.stop()

default_kv = {'kkk-%s' % i : 'vvv-%s' % i for i in range(10)}

def getconn():
    for r in all_redis:
        c = redis.Redis(r.host(), r.port())
        c.flushdb()

    r = redis.Redis(nc.host(), nc.port())
    return r




#!/usr/bin/env python
#coding: utf-8
#file   : test_mget.py
#author : ning
#date   : 2014-04-01 13:15:48

import os
import re
import commands

ports = [
    4001,  # before improve
    4000,  # after improve
    2000   # redis
]

def system(cmd):
    return commands.getoutput(cmd)

def extra(regex, text):
    match = re.search(regex, text, re.DOTALL)
    if match:
        return match.group(1)

def testit():
    for mget_size in [10, 100, 1000, 10000]:
        for port in ports:
            cnt = 100*1000 / mget_size
            clients = 50
            if mget_size == 10000:
                clients = 2
            cmd = 'cd /home/ning/xredis/deploy-srcs/redis-2.8.3/src && ./redis-benchmark.%d -n %d -p %d -t mget  -r 1000000000 -c %d' % (mget_size, cnt, port, clients)
            #print cmd
            rst = system(cmd)

            #100.00% <= 2 milliseconds
            #28089.89 requests per second
            rtime = extra('100.00% <= (\d+) milliseconds', rst)
            qps = extra('([\.\d]+) requests per second', rst)

            print 'mget_size=%d on %d: pqs: %s, rtime: %s' % (mget_size, port, qps, rtime)

testit()



#!/usr/bin/env python
#coding: utf-8

from common import *

all_redis = [
    RedisServer('127.0.0.1', 2100, '/tmp/r/redis-2100/',
                CLUSTER_NAME, 'redis-2100', auth = 'hellopasswd'),
    RedisServer('127.0.0.1', 2101, '/tmp/r/redis-2101/',
                CLUSTER_NAME, 'redis-2101', auth = 'hellopasswd'),
]

nc = NutCracker('127.0.0.1', 4100, '/tmp/r/nutcracker-4100', CLUSTER_NAME,
                all_redis, mbuf=mbuf, verbose=nc_verbose,
                redis_auth = 'hellopasswd')

nc_badpass = NutCracker('127.0.0.1', 4101, '/tmp/r/nutcracker-4101', CLUSTER_NAME,
                        all_redis, mbuf=mbuf, verbose=nc_verbose,
                        redis_auth = 'badpasswd')
nc_nopass = NutCracker('127.0.0.1', 4102, '/tmp/r/nutcracker-4102', CLUSTER_NAME,
                       all_redis, mbuf=mbuf, verbose=nc_verbose)

def setup():
    print 'setup(mbuf=%s, verbose=%s)' %(mbuf, nc_verbose)
    for r in all_redis + [nc, nc_badpass, nc_nopass]:
        r.clean()
        r.deploy()
        r.stop()
        r.start()

def teardown():
    for r in all_redis + [nc, nc_badpass, nc_nopass]:
        assert(r._alive())
        r.stop()

default_kv = {'kkk-%s' % i : 'vvv-%s' % i for i in range(10)}

def getconn():
    r = redis.Redis(nc.host(), nc.port())
    return r

'''

cases:


redis       proxy       case
1           1           test_auth_basic
1           bad         test_badpass_on_proxy
1           0           test_nopass_on_proxy
0           0           already tested on other case
0           1

'''

def test_auth_basic():
    # we hope to have same behavior when the server is redis or twemproxy
    conns = [
         redis.Redis(all_redis[0].host(), all_redis[0].port()),
         redis.Redis(nc.host(), nc.port()),
    ]

    for r in conns:
        assert_fail('NOAUTH|operation not permitted', r.ping)
        assert_fail('NOAUTH|operation not permitted', r.set, 'k', 'v')
        assert_fail('NOAUTH|operation not permitted', r.get, 'k')

        # bad passwd
        assert_fail('invalid password', r.execute_command, 'AUTH', 'badpasswd')

        # everything is ok after auth
        r.execute_command('AUTH', 'hellopasswd')
        r.set('k', 'v')
        assert(r.ping() == True)
        assert(r.get('k') == 'v')

        # auth fail here, should we return ok or not => we will mark the conn state as not authed
        assert_fail('invalid password', r.execute_command, 'AUTH', 'badpasswd')

        assert_fail('NOAUTH|operation not permitted', r.ping)
        assert_fail('NOAUTH|operation not permitted', r.get, 'k')

def test_nopass_on_proxy():
    r = redis.Redis(nc_nopass.host(), nc_nopass.port())

    # if you config pass on redis but not on twemproxy,
    # twemproxy will reply ok for ping, but once you do get/set, you will get errmsg from redis
    assert(r.ping() == True)
    assert_fail('NOAUTH|operation not permitted', r.set, 'k', 'v')
    assert_fail('NOAUTH|operation not permitted', r.get, 'k')

    # proxy has no pass, when we try to auth
    assert_fail('Client sent AUTH, but no password is set', r.execute_command, 'AUTH', 'anypasswd')
    pass

def test_badpass_on_proxy():
    r = redis.Redis(nc_badpass.host(), nc_badpass.port())

    assert_fail('NOAUTH|operation not permitted', r.ping)
    assert_fail('NOAUTH|operation not permitted', r.set, 'k', 'v')
    assert_fail('NOAUTH|operation not permitted', r.get, 'k')

    # we can auth with bad pass (twemproxy will say ok for this)
    r.execute_command('AUTH', 'badpasswd')
    # after that, we still got NOAUTH for get/set (return from redis-server)
    assert(r.ping() == True)
    assert_fail('NOAUTH|operation not permitted', r.set, 'k', 'v')
    assert_fail('NOAUTH|operation not permitted', r.get, 'k')

def setup_and_wait():
    time.sleep(60*60)



#!/usr/bin/env python
#coding: utf-8

import os
import sys
import redis
import memcache

PWD = os.path.dirname(os.path.realpath(__file__))
WORKDIR = os.path.join(PWD,  '../')
sys.path.append(os.path.join(WORKDIR, 'lib/'))
sys.path.append(os.path.join(WORKDIR, 'conf/'))
import conf

from server_modules import *
from utils import *

CLUSTER_NAME = 'ntest'
all_mc= [
        Memcached('127.0.0.1', 2200, '/tmp/r/memcached-2200/', CLUSTER_NAME, 'mc-2200'),
        Memcached('127.0.0.1', 2201, '/tmp/r/memcached-2201/', CLUSTER_NAME, 'mc-2201'),
    ]

nc_verbose = int(getenv('T_VERBOSE', 4))
mbuf = int(getenv('T_MBUF', 512))
large = int(getenv('T_LARGE', 1000))

nc = NutCracker('127.0.0.1', 4100, '/tmp/r/nutcracker-4100', CLUSTER_NAME,
                all_mc, mbuf=mbuf, verbose=nc_verbose, is_redis=False)

def setup():
    for r in all_mc:
        r.deploy()
        r.stop()
        r.start()

    nc.deploy()
    nc.stop()
    nc.start()

def teardown():
    for r in all_mc:
        r.stop()
    assert(nc._alive())
    nc.stop()

def getconn():
    host_port = '%s:%s' % (nc.host(), nc.port())
    return memcache.Client([host_port])

def test_basic():
    conn = getconn()
    conn.set('k', 'v')
    assert('v' == conn.get('k'))

    conn.set("key", "1")
    for i in range(10):
        conn.incr("key")
        assert(str(i+2) == conn.get('key'))

    conn.delete("key")
    assert(None == conn.get('key'))

default_kv = {'kkk-%s' % i :'vvv-%s' % i for i in range(10)}
def test_mget_mset(kv=default_kv):
    conn = getconn()
    conn.set_multi(kv)
    keys = sorted(kv.keys())

    assert(conn.get_multi(keys) == kv)
    assert(conn.gets_multi(keys) == kv)

    #del
    conn.delete_multi(keys)
    #mget again
    vals = conn.get_multi(keys)
    assert({} == vals)

def test_mget_mset_large():
    for cnt in range(179, large, 179):
        #print 'test', cnt
        kv = {'kkk-%s' % i :'vvv-%s' % i for i in range(cnt)}
        test_mget_mset(kv)

def test_mget_mset_key_not_exists(kv=default_kv):
    conn = getconn()
    conn.set_multi(kv)

    keys = kv.keys()
    keys2 = ['x-'+k for k in keys]
    keys = keys + keys2
    random.shuffle(keys)

    for i in range(2):
        #mget to check
        vals = conn.get_multi(keys)
        for i, k in enumerate(keys):
            if k in kv:
                assert(kv[k] == vals[k])
            else:
                assert(k not in vals)

    #del
    conn.delete_multi(keys)
    #mget again
    vals = conn.get_multi(keys)
    assert({} == vals)

